import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

/*
 * Graphical game board
 */
public class Board extends JPanel implements KeyListener  {
	private static final long serialVersionUID = 991601048688376796L;
	
	// Game board parameters
	public static int width = 10;
	public static int height = 20;
	
	// Graphical display parameters
	private static int squareSize = 26;
	private int padding = 10;

	// Pseudo-random generator initialized at the seed
	private PiecesManager piecesManager;
	
	// Colors for the pieces
	public static GradientPaint ColorEmpty = new GradientPaint(0, 0, new Color(40,10,60,220), 0, squareSize/2, new Color(40,10,60,220), true);
	public static GradientPaint ColorI = new GradientPaint(0, 0, new Color(105,255,105), 0, squareSize/2, new Color(155,255,155), true);
	public static GradientPaint ColorJ = new GradientPaint(0, 0, new Color(50,50,155), 0, squareSize/2, new Color(50,50,195), true);
	public static GradientPaint ColorL = new GradientPaint(0, 0, new Color(125,50,125), 0, squareSize/2, new Color(165,50,165), true);
	public static GradientPaint ColorO = new GradientPaint(0, 0, new Color(200,200,100), 0, squareSize/2, new Color(245,245,100), true);
	public static GradientPaint ColorS = new GradientPaint(0, 0, new Color(100,190,205), 0, squareSize/2, new Color(100,190,255), true);
	public static GradientPaint ColorT = new GradientPaint(0, 0, new Color(255,20,20), 0, squareSize/2, new Color(205,20,20), true);
	public static GradientPaint ColorZ = new GradientPaint(0, 0, new Color(255,255,255), 0, squareSize/2, new Color(215,215,215), true);
	public static GradientPaint ColorP = new GradientPaint(0, 0, new Color(105,105,105), 0, squareSize/2, new Color(75,75,75), true);
	public static GradientPaint ColorShadow = new GradientPaint(0, 0, new Color(200,200,200,25), 0, squareSize/2, new Color(245,245,245,75), true);
	
	// Logic for the board and the moving piece
	private Piece.Type[][] board = new Piece.Type[height][width];
	private Piece currentPiece;
	private Piece ghost;
	private NextPiece nextPiece;
	private Game game;
	private Grid grid = new Grid(width, height);
	private Random rand = new Random(); 
	
	// Logic for the lines and score
	private ScoreBoard scoreBoard;
	private int linesCount = 0;
	private int score = 0;
	private int combo = 0;
	private PriorityQueue<Integer> penalties = new PriorityQueue<Integer>();

	/*
	 * Initialization of the board object (empty board, seed, new piece...)
	 */
	public Board(PiecesManager _piecesManager, Game _game, ScoreBoard _scoreBoard) {
		piecesManager = _piecesManager;
		game = _game;
		scoreBoard = _scoreBoard;
		    	
		this.setBounds(0, 0, width * squareSize+2*padding+600, height * squareSize+2*padding);
		this.setOpaque(false);		

		this.setLayout(null);
		nextPiece = new NextPiece();
		this.add(nextPiece);
		
		clear();
		newPiece();
		updateGhost();

		// Initializes the listener for the keyboard
		addKeyListener(this);
	}

	/*
	 * Draws the board and the current piece in overlay
	 */
    public void paintComponent(Graphics g){
        Graphics2D g2d = (Graphics2D)g;         
        
    	grid.setContent(board);
    	grid.print(g2d);

		currentPiece.print(g2d,squareSize,padding);
		ghost.print(g2d,squareSize,padding);
    } 

    /*
     * Adds one line to the line counter and handles the leveling up
     */
    private void newLine() {
    	linesCount ++;
    	
    	if (linesCount % 10 == 0) {
    		piecesManager.levelup();
    	}
    }

    /*
     * Resets the board to an empty board
     */
    public void clear() {
    	for(int x=0;x<width;x++){
			for(int y=0;y<height;y++){
				board[y][x] = Piece.Type.empty;
			}		
		}
    }
    
    /*
     *  Checks if a given position of the board is empty
     */
    public boolean isEmpty(int x, int y){
    	return (x >= 0 && y >= 0 && x < width && y < height && (board[y][x] == Piece.Type.empty));    
    }

    /*
     *  Checks the whole board for tetris lines and corrects it
     *  Creates a copy of the board, might need some optimizing
     */
    public void checkBoard() {
    	// the new copy of the board after the complete lines have been deleted
    	Piece.Type[][] newBoard = new Piece.Type[height][width];
    	int ny = height-1;
    	
    	int nbLines = 0;
    	
    	// Copy from bottom to top
    	for(int y=height-1;y>=0;y--){
			newBoard[ny] = board[y];
			
    		Boolean tetris = true;
    		
    		for(int x=0;x<width;x++){
				if (board[y][x] == Piece.Type.empty) {
					tetris = false;
				}
			}		

    		if (tetris){
    			newLine();
    			score += 5 + nbLines;
    			nbLines ++;
    		}
    		else {
	    		ny--;			
    		}
		}
    	
    	// Complete the board, in case lines have been erased
    	for(int y=ny;y>=0;y--){
    		for(int x=0;x<width;x++){
    			newBoard[y][x] = Piece.Type.empty;
			}
		}
    	
    	board = newBoard;

		Sounds sounds = new Sounds();
		if (nbLines > 0){
			combo ++;
			int penalties = (int)((nbLines-1) + Math.floor(combo/2));
			sendPenalties(penalties);
			if (nbLines == 4) {
				sounds.holyshit();
			}
			else if (nbLines == 3) {
				sounds.killingspree();
			}
			else if (nbLines == 2) {
				sounds.rampage();
			}
			else {
				sounds.roar();
			}
		}
		else {
			if (combo > 0) {
				sounds.combob();
			}
			else {
				sounds.meow();
			}
			
			combo = 0;
			if (penalties.size() > 0){
				cashPenalties();
			}
		}

		scoreBoard.upStats(linesCount, score, combo);
    }
    
    /*
     * Fixes the type of a given position on the board
     */
    public void setType(int x, int y, Piece.Type t){
    	board[y][x] = t;    
    }

    /*
     *  Cash the buffered penalties
     */
    private void cashPenalties(){
    	Piece.Type[][] newBoard = new Piece.Type[height][width];
    	Piece.Type[] blocklines = new Piece.Type[width];
    	for (int i=0;i<width;i++){
    		blocklines[i] = Piece.Type.P;
    	}
    	
    	int y = height - 1;
        while (penalties.size() != 0)
        {
            int hole = penalties.remove();
            Piece.Type[] line = blocklines.clone();
            line[hole] = Piece.Type.empty;
            
            if (y>=0) {
            	newBoard[y] = line;
            }
            y--;
        }
        
        for (int j=height-1; j>=0; j--){
            if (y>=0) {
            	newBoard[y] = board[j];
            }
            y--;        	
        }
        
        board = newBoard;
		scoreBoard.upPenalties(penalties.size());
    }
    
    /*
     * Handles the creation of a new piece using the piece manager
     */
    public void newPiece(){
    	piecesManager.newType();
    	Piece.Type type = piecesManager.current();
    	
		try {
			currentPiece = new Piece(this, type, width / 2, 1);
			ghost = new Piece(this, type, width / 2, 1);
			ghost.setGhost();
			updateGhost();
		} catch (Exception e) {
			loose();
		}
		
		nextPiece.switchPiece(piecesManager.nextType());
		
		game.broadcastBoard(board);
    }
    
    /*
     * Handles a tic of the clock
     */
    public void tic() {
    	currentPiece.goDown();
		this.repaint();
    }

    /*
     * Handles the loosing of the game
     */
    public void loose() {
    	Piece.Type[][] looseboard = BoardFromText("...ppp....-..pooop...-.pooojop..-poooooop..-pooppoop..-.pop.pop..-..pp..p...-..........-.ttt..ttt.-.t....t...-.t.t..t.t.-.ttt..ttt.-..........-...p..pp..-..pop.pop.-..pooppoop-..poooooop-..pojooop.-...pooop..-....ppp...");

    	currentPiece = new Piece(Piece.Type.empty, -100,-100);
    	board = looseboard;
    	this.repaint();
    	
    	System.out.println("O NOEZ, I JUST LOST TEH GAMZ !!!");
    	piecesManager.stop();
    	removeKeyListener(this);
    	game.broadcastBoard(looseboard);

		Runnable doWorkRunnable = new Runnable() {
		    public void run() {  game.loose(); }
		};
		SwingUtilities.invokeLater(doWorkRunnable);
    }
    
	/*
	 * Handles the taking of penalties
	 */
	public void takePenalties(int n){
		int hole = rand.nextInt(width);

		Sounds sounds = new Sounds();
		sounds.turkey();
		
		for (int i=0;i<n;i++){			
			penalties.add(hole);
		}
		// TODO
		scoreBoard.upPenalties(penalties.size());
	}

	/*
	 * Reads the content of the buffer (waiting for it to be non-null)
	 */
	private Piece.Type[][] BoardFromText(String str) {
		String[] s = str.split("-");
		
		final Piece.Type[][] board = new Piece.Type[Board.height][Board.width];

		
		for (int y=0;y<height;y++){
			for (int x=0;x<width;x++){
				Piece.Type p = Piece.Type.empty;

	            switch (s[y].charAt(x)) {
		            case 'i' :  p = Piece.Type.I;  break;
		            case 'j' :  p = Piece.Type.J;  break;
		            case 'l' :  p = Piece.Type.L;  break;
		            case 'o' :  p = Piece.Type.O;  break;
		            case 's' :  p = Piece.Type.S;  break;
		            case 't' :  p = Piece.Type.T;  break;
		            case 'z' :  p = Piece.Type.Z;  break;
		            case 'p' :  p = Piece.Type.P;  break;
		            default: break;
	            }
	            
				board[y][x] = p;
			}
		}

		return board;
	}
	
    
	/*
	 * Handles the sending of penalties, clearing first the line buffer
	 */
	public void sendPenalties(int n){        
        while (penalties.size() != 0 && n > 0)
        {
            penalties.remove();
            n--;
        }
        
        if (n>0){
        	game.sendPenalties(n);
        }

		scoreBoard.upPenalties(penalties.size());
	}
    
	/*
	 * updates the ghost
	 */
	public void updateGhost(){
		ghost.setCoord(currentPiece.x, currentPiece.y);
		ghost.goMostDown();
	}
	
	/*
	 * Event handler for the validation a message with the key
	 */
    public void keyPressed(KeyEvent e) {
        int key = e.getKeyCode();

        if (key == KeyEvent.VK_UP) {
			if (currentPiece.rotate()) {
				ghost.rotate();
			}
			updateGhost();
        }
        else if (key == KeyEvent.VK_LEFT) {
			currentPiece.goLeft();
			updateGhost();
        }
        else if (key == KeyEvent.VK_RIGHT) {
			currentPiece.goRight();
			updateGhost();
        }
        else if (key == KeyEvent.VK_DOWN) {
			currentPiece.goDown();
			updateGhost();
        }
        else if (key == KeyEvent.VK_SPACE) {
			currentPiece.goMostDown();
        }/*
        else if (key == KeyEvent.VK_K) {
			combo ++;
			scoreBoard.upStats(linesCount, score, combo);
        }
        else if (key == KeyEvent.VK_P) {
			game.sendPenalties(1);
        }
        else if (key == KeyEvent.VK_O) {
        	penalties.clear();
        }
        else if (key == KeyEvent.VK_C) {
			clear();
        }*/
        else if (key == KeyEvent.VK_L) {
			game.broadcastBoard(BoardFromText("....p...p.-...pzp.pp.-...pzzpzp.-...pszzsp.-...pzzzzp.-...pzttzp.-pppppppp..-pzzzzzzzp.-.ppzzzzzp.-...pzzzzp.-...pzzzzp.-...pzzzzp.-...pzzzzp.-...pzzzzp.-...pzzzzp.-...pzzzzp.-...pzzzzp.-...pzzzzp.-...pzzzzp.-...pzzzzp."));
        }
		this.repaint();
    }
    public void keyTyped(KeyEvent e) {}
    public void keyReleased(KeyEvent e) {}

}
