import java.awt.GradientPaint;
import java.awt.Graphics2D;
import java.util.Vector;

import javax.swing.SwingUtilities;

/*
 * Logic for the tetris piece
 */
public class Piece {
	/*
	 * Different types of pieces, including empty type
	 */
	public static enum Type {
	    I, J, L, O, S, T, Z, P, empty, ghost 
	}
	
	// Logical components
	private Board board;
	
	// Piece parameters
	public Type type;
	public int x,y;
	private Vector<int[]> squares = new Vector<int[]>();
	
	/*
	 * This constructor generates a piece of a given type, at a given position. Might throw an exception if not possible (game lost)
	 */
	public Piece(Board _board, Type _type, int _x, int _y) throws Exception {
		board = _board;
		type = _type;
		x = _x;
		y = _y;
		
		constructSquares();
		
        // Throws an exception if creation not possible
        if (! dontCreateCollision(x,y,squares)){
        	throw new Exception("LOST TEH GAMZ");
        }
	}
	
	/*
	 * This creates a piece WITHOUT LOGIC for the sole purpose of printing (nextPiece)
	 */
	public Piece(Type _type, int _x, int _y) {
		type = _type;
		x = _x;
		y = _y;
		constructSquares();
	}
	
	/*
	 * Constructs the whole piece from its type
	 */
	private void constructSquares(){
		squares = new Vector<int[]>();

		addSquare(0,0);

        switch (type) {
	        case I : addSquare(0,1); addSquare(0,-1); addSquare(0,2); break;
	        case J : addSquare(-1,0); addSquare(-1,-1); addSquare(1,0); break;
	        case L : addSquare(-1,0); addSquare(1,-1); addSquare(1,0); break;
	        case O : addSquare(0,-1); addSquare(1,0); addSquare(1,-1); break;
	        case S : addSquare(-1,0); addSquare(0,-1); addSquare(1,-1); break;
	        case T : addSquare(-1,0); addSquare(1,0); addSquare(0,-1); break;
	        case Z : addSquare(-1,-1); addSquare(0,-1); addSquare(1,0); break;
        default:  break;
        }		
	}

	/*
	 * Adds a square to the manipulated piece
	 */
	private void addSquare(int i, int j) {
		int[] pair = {i,j};
		squares.add(pair);
	}

	/*
	 * Draws the piece on the graphical board given
	 */
	public void print (Graphics2D g2d, int squareSize, int padding) {
        GradientPaint gp = Board.ColorEmpty;
        
        // Sets the color from board parameters
        switch (type) {
	        case I : gp = Board.ColorI; break;
	        case J : gp = Board.ColorJ; break;
	        case L : gp = Board.ColorL; break;
	        case O : gp = Board.ColorO; break;
	        case S : gp = Board.ColorS; break;
	        case T : gp = Board.ColorT; break;
	        case Z : gp = Board.ColorZ; break;
	        case ghost : gp = Board.ColorShadow; break;
        }

        g2d.setPaint(gp);

        for(int i=0;i<squares.size();i++)
        {
        	int[] square = squares.get(i);
            g2d.fillRect(squareSize*(x + square[0])+padding+1, squareSize*(y + square[1])+padding+1, squareSize-1, squareSize-1);        	
        }
	}
	
	/*
	 * Checks if the given position for the piece is valid, that is to say doenst create a collision
	 */
	private Boolean dontCreateCollision(int nx, int ny, Vector<int[]> nSquares){
		Boolean r = true;
        for(int i=0;i<nSquares.size();i++)
        {
        	int[] square = nSquares.get(i);
        	r = r && board.isEmpty(nx + square[0], ny + square[1]);    	
        }
        
		return r;
	}

	/* 
	 * Gives to this piece the ghost properties
	 */
	public void setGhost(){
		type = Type.ghost;
	}
	
	/*
	 * Changes the coordinates of the piece manually
	 */
	public void setCoord(int _x, int _y) {
		x = _x;
		y = _y;
	}
	
	/*
	 * Rotates the piece if possible, returns true if possible
	 */
	public Boolean rotate() {
		Vector<int[]> nsquares = new Vector<int[]>();
        for(int i=0;i<squares.size();i++)
        {
        	int[] square = squares.get(i);
        	int[] newsquare = {-square[1], square[0]};
        	nsquares.add(newsquare);
        }
        
		if (dontCreateCollision(x,y,nsquares) || type == Type.ghost) {
			squares = nsquares;
			return true;
		}
		return false;
	}	

	/*
	 * Moves one block to the left if possible
	 */
	public void goLeft() {
		if (dontCreateCollision(x-1,y,squares)) {
			x--;	
		}
	}

	/*
	 * Moves one block to the right if possible
	 */
	public void goRight() {
		if (dontCreateCollision(x+1,y,squares)) {
			x++;	
		}	
	}

	/*
	 * Moves one block down if possible
	 */
	public void goDown(){
		if (dontCreateCollision(x,y+1,squares)) {
			y++;	
		}
		else {
			fusionWithBoard();
		}			
	}

	/*
	 * Goes down all the way 
	 */
	public void goMostDown(){
		while (dontCreateCollision(x,y+1,squares)) {
			y++;	
		}
		fusionWithBoard();
	}
	
	/*
	 * Paste this piece to the solid board, so it becomes part of the board. Launches the creation of a new piece
	 */
	private void fusionWithBoard(){			
		if (type != Type.ghost){
	        for(int i=0;i<squares.size();i++)
	        {
	        	int[] square = squares.get(i);
	        	int ny = y + square[1];
	        	
	        	board.setType(x + square[0], ny, type);  
	        }
	
	        // Check the board for tetris and launches a new piece
	        board.checkBoard();
	        board.newPiece();
		}
	}
}
