import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

public class Screen extends JFrame implements WindowListener {
	private static final long serialVersionUID = 7332873980574109461L;
	
	// Logical interaction
	private ServerInteraction serverInteraction;
	private Game game;
	
	// Settings
	private int width = 1024;
	private int height = 768;

	// Image background panel parameters
	private JScrollPane scrollPane;	
	private JPanel panel;	
	private ImageIcon background;

	// Content
	private PlayersList playersList;
	private Authentification authentification;
	private StartButton startButton;
	private Chat chat;
	private DisconnectButton disconnectButton;

	/* 
	 * Initializes the window
	 */
	public Screen(Game _game, PlayersList _playersList)
	{
		this.addWindowListener(this);
		
		// Initializing logical components
		playersList = _playersList;
		game = _game;
		
		serverInteraction = new ServerInteraction(this, game, playersList);
		authentification = new Authentification(this, serverInteraction, game);
		startButton = new StartButton(serverInteraction);
		chat = new Chat(serverInteraction);
		disconnectButton = new DisconnectButton(serverInteraction, this, game);
		
		// Initializes the background pannel
		initializesBackground();

		// Initializes the content pannel
		panel.setLayout(null);
		
		// Initializes the display
		buildPlayersList();
		buildAuthentification();
		buildStart();
		buildChat();
		buildGame();
		buildDisconnect();
	}
	

	/* 
	 * Sets the parameters of the window and refreshed it
	 */
	public void refreshScreen(Game _game, PlayersList _playersList)
	{
		playersList.setVisible(false);
		
		// Initializing logical components
		playersList = _playersList;
		game = _game;

		serverInteraction = new ServerInteraction(this, game, playersList);
		authentification = new Authentification(this, serverInteraction, game);
		startButton = new StartButton(serverInteraction);
		chat = new Chat(serverInteraction);
		disconnectButton = new DisconnectButton(serverInteraction, this, game);
		
		buildPlayersList();
		buildAuthentification();
		buildStart();
		buildChat();
		buildGame();
		buildDisconnect();
		
		displayAuth();
	}

	/*
	 * Builds the picture background of the screen
	 */
	private void initializesBackground(){
		background = new ImageIcon("background.jpg");

		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setResizable(false);
		
		setSize(width, height);
		setVisible(true);
 
		panel = new JPanel()
		{
			private static final long serialVersionUID = 8943024746013222195L;

			protected void paintComponent(Graphics g)
			{
				g.drawImage(background.getImage(), 0, 0, scrollPane.getWidth(), scrollPane.getHeight(),  null); 
				super.paintComponent(g);
			}
		};
		
		panel.setOpaque( false );
		panel.setPreferredSize( new Dimension(width / 2, height / 2) );
		scrollPane = new JScrollPane( panel );
		getContentPane().add( scrollPane );		
	}


	
	
	
// Initializing graphical interface
	
	/* 
	 * Initializes the display of the players list
	 */
	private void buildPlayersList() {
		playersList.setBounds(20, 400, 150, 300);
		playersList.setVisible(false);
		panel.add (playersList);
	}

	/* 
	 * Initializes the display of the connection form
	 */
	private void buildAuthentification() {
		authentification.setBounds(350, 200, 300, 200);
		authentification.setVisible(true);
		panel.add (authentification);
	}

	/* 
	 * Initializes the display of the start button
	 */
	private void buildStart(){
		startButton.setBounds(80, 90, 400, 300);
		startButton.setVisible(false);
		panel.add(startButton);
	}

	/* 
	 * Initializes the display of the chat panel
	 */
	private void buildChat() {
		chat.setBounds(250, 400,700, 250);
		chat.setVisible(false);
		panel.add(chat);
	}

	/* 
	 * Initializes the display of the game panel
	 */
	private void buildGame() {
		game.setBounds(200, 100,800, 600);
		game.setVisible(false);
		panel.add(game);
	}

	/* 
	 * Initializes the display of the disconnect button
	 */
	private void buildDisconnect(){
		disconnectButton.setBounds(800, 700, 200, 20);
		disconnectButton.setVisible(false);
		panel.add( disconnectButton );
	}

	
	
	
// Visibility functions	
	
	/*
	 * Displays the authentification screen with editable server fields (reset)
	 */
	public void displayAuth(){
		playersList.setVisible(false);
		authentification.setVisible(true);
		authentification.setServerEditable(true);
		startButton.setVisible(false);
		chat.setVisible(false);
		disconnectButton.setVisible(false);
	}

	/*
	 * Displays the chat screen
	 */
	public void displayChat(){
		playersList.setVisible(true);	
		authentification.setVisible(false);
		chat.setVisible(true);
		chat.emptyChat();
		disconnectButton.setVisible(true);
	}
	
	/*
	 * Displays the game screen
	 */
	public void displayGame(){
		playersList.setVisible(true);
		authentification.setVisible(false);
		startButton.setVisible(false);
		chat.setVisible(false);	
		disconnectButton.setVisible(false); // ?
	}
	
	
	
// Various interaction functions	
	
	/*
	 * Refresh the leadership status and the elements depending on it (startButton)
	 */
	public void refreshLeadership() {
		startButton.refresh();
	}
	
	/*
	 * Adds a line to the chat
	 */
	public void addChatLine(String name, String msg) {
		chat.addLine(name, msg);
	}
	
	// Window listeners
	public void windowClosing(WindowEvent e) {
    	game.loose();
	}
	
    public void windowClosed(WindowEvent e) {}

    public void windowOpened(WindowEvent e) {}

    public void windowIconified(WindowEvent e) {}

    public void windowDeiconified(WindowEvent e) {}

    public void windowActivated(WindowEvent e) {}

    public void windowDeactivated(WindowEvent e) {}

    public void windowGainedFocus(WindowEvent e) {}

    public void windowLostFocus(WindowEvent e) {}

    public void windowStateChanged(WindowEvent e) {}
}
